VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "HDSLN"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'=====
'HDSLN
'=====
'
'A "get hardware drive serial number" class.
'
'Has one (default) property: Value As String
'
'Since it is the default property and you seldom need to query it
'twice, you can often call it simply as in:
'
'       Serial = New HDSLN
'
'Or more verbosely as:
'
'       Dim HDSLN As New HDSLN
'
'       Serial = HDSLN.Value
'
'Or even:
'
'       With New HDSLN
'           Serial = .Value
'       End With
'
'First tries to get the "Windows Genuine Value" HDSLN from the
'registry.  If that fails, then it tries to get it from
'PhysicalDrive0 via a DeviceIoControl call.  However, on an NT
'system after XP this requires elevated rights (full admin
'rights).
'
'The returned value is the drive serial number or an empty string
'value ("") if it cannot be obtained.
'
'
'-----
'NOTE:
'-----
'
'   Not tested in a Win9X system yet, so this may require more
'   work before it is reliable.  Such systems are seldom part of
'   the "Genuine Advantage" program so they need to get data from
'   the device.
'

Private Const VER_PLATFORM_WIN32_WINDOWS = 1
Private Const VER_PLATFORM_WIN32_NT = 2

Private Const DFP_RECEIVE_DRIVE_DATA = &H7C088

Private Const FILE_SHARE_READ = &H1
Private Const FILE_SHARE_WRITE = &H2
Private Const FILE_SHARE_DELETE = &H4
Private Const GENERIC_READ = &H80000000
Private Const GENERIC_WRITE = &H40000000
Private Const OPEN_EXISTING = 3
Private Const CREATE_NEW = 1
Private Const INVALID_HANDLE_VALUE = -1

Private Type OSVERSIONINFO
    dwOSVersionInfoSize As Long
    dwMajorVersion As Long
    dwMinorVersion As Long
    dwBuildNumber As Long
    dwPlatformId As Long
    szCSDVersion As String * 128
End Type

Private Type IDEREGS
    bFeaturesReg As Byte
    bSectorCountReg As Byte
    bSectorNumberReg As Byte
    bCylLowReg As Byte
    bCylHighReg As Byte
    bDriveHeadReg As Byte
    bCommandReg As Byte
    bReserved As Byte
End Type

Private Type SENDCMDINPARAMS
    cBufferSize As Long
    irDriveRegs As IDEREGS
    bDriveNumber As Byte
    bReserved(1 To 3) As Byte
    dwReserved(1 To 4) As Long
End Type

Private Type DRIVERSTATUS
    bDriveError As Byte
    bIDEStatus As Byte
    bReserved(1 To 2) As Byte
    dwReserved(1 To 2) As Long
End Type

Private Type SENDCMDOUTPARAMS
    cBufferSize As Long
    DStatus As DRIVERSTATUS
    bBuffer(1 To 512) As Byte
End Type

Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long

Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" ( _
    ByVal lpFileName As String, _
    ByVal dwDesiredAccess As Long, _
    ByVal dwShareMode As Long, _
    ByVal lpSecurityAttributes As Long, _
    ByVal dwCreationDisposition As Long, _
    ByVal dwFlagsAndAttributes As Long, _
    ByVal hTemplateFile As Long) As Long

Private Declare Function DeviceIoControl Lib "kernel32" ( _
    ByVal hDevice As Long, _
    ByVal dwIoControlCode As Long, _
    ByRef InBuffer As Any, _
    ByVal nInBufferSize As Long, _
    ByRef OutBuffer As Any, _
    ByVal nOutBufferSize As Long, _
    ByRef BytesReturned As Long, _
    ByVal lpOverlapped As Long) As Long

Private Declare Function GetVersionEx Lib "kernel32" Alias "GetVersionExA" ( _
    ByRef lpVersionInformation As OSVERSIONINFO) As Long

Private Declare Function RegCloseKey Lib "advapi32" (ByVal hKey As Long) As Long

Private Declare Function RegOpenKeyEx Lib "advapi32" _
    Alias "RegOpenKeyExW" ( _
    ByVal hKey As Long, _
    ByVal lpSubKey As Long, _
    ByVal ulOptions As Long, _
    ByVal samDesired As Long, _
    ByRef phkResult As Long) As Long

Private Declare Function RegQueryValueEx Lib "advapi32" _
    Alias "RegQueryValueExW" ( _
    ByVal hKey As Long, _
    ByVal lpValueName As Long, _
    ByVal lpReserved As Long, _
    ByRef lpType As Long, _
    ByVal lpData As Long, _
    ByRef lpcbData As Long) As Long

Private mvarPlatform As Long

'Marked as Default Member!
Public Property Get Value() As String
Attribute Value.VB_UserMemId = 0
    On Error Resume Next
    Value = GetFromRegistry()
    If Err Then
        Err.Clear
        Value = GetFromDevice(0)
    End If
End Property

Private Function GetFromDevice(Optional ByVal Drive As Byte = 0) As String
    'Drive is the IDE drive 0 through 3.
    Dim hDrive As Long
    Dim bufferRequest As SENDCMDINPARAMS
    Dim buFferResponse As SENDCMDOUTPARAMS
    Dim Result As Long
    Dim cbReturned As Long
    Dim I As Integer
    Dim S As String

    If Drive > 3 Then Err.Raise &H8004F600, TypeName(Me), "Drive must be 0 through 3"
    
    hDrive = INVALID_HANDLE_VALUE
    Select Case mvarPlatform
        Case VER_PLATFORM_WIN32_NT
            hDrive = CreateFile("\\.\PhysicalDrive" & CStr(Drive), _
                                GENERIC_READ Or GENERIC_WRITE, _
                                FILE_SHARE_READ Or FILE_SHARE_WRITE Or FILE_SHARE_DELETE, _
                                0, OPEN_EXISTING, 0, 0)
        Case VER_PLATFORM_WIN32_WINDOWS
            hDrive = CreateFile("\\.\Smartvsd", 0, 0, 0, CREATE_NEW, 0, 0)
        Case Else
            Err.Raise &H8004F602, TypeName(Me), "Illegal platform (only NT or 9X)"
    End Select
    
    If hDrive = INVALID_HANDLE_VALUE Then
        Err.Raise &H8004F604, _
                  TypeName(Me), _
                  "CreateFile failed, system error " & CStr(Err.LastDllError)
    End If

    With bufferRequest
        .bDriveNumber = Drive
        .cBufferSize = 512
        With .irDriveRegs
            If (Drive And 1) Then
                .bDriveHeadReg = &HB0
            Else
                .bDriveHeadReg = &HA0
            End If
            .bCommandReg = &HEC
            .bSectorCountReg = 1
            .bSectorNumberReg = 1
        End With
    End With

    Result = DeviceIoControl(hDrive, _
                             DFP_RECEIVE_DRIVE_DATA, _
                             bufferRequest, _
                             LenB(bufferRequest), _
                             buFferResponse, _
                             LenB(buFferResponse), _
                             cbReturned, _
                             0)
    CloseHandle hDrive
    If Result = 0 Then
        Err.Raise &H8004F606, _
                  TypeName(Me), _
                  "DeviceIoControl failed, system error " & CStr(Err.LastDllError)
    Else
        For I = 21 To 40 Step 2
            If buFferResponse.bBuffer(I + 1) = 0 Then Exit For
            S = S & Chr$(buFferResponse.bBuffer(I + 1))
            If buFferResponse.bBuffer(I) = 0 Then Exit For
            S = S & Chr$(buFferResponse.bBuffer(I))
        Next I
        GetFromDevice = Trim$(S)
    End If
End Function

Private Function GetFromRegistry() As String
    Const HKLM = &H80000002
    Const SubKey = "SOFTWARE\Microsoft\Windows Genuine Advantage"
    Const KEY_READ As Long = &H20019
    Const ValueName = "HDSLN"
    Dim Result As Long
    Dim hOpenKey As Long
    Dim lngDataSize As Long
    Dim S As String
    Dim lngType As Long
    
    Result = RegOpenKeyEx(HKLM, StrPtr(SubKey), 0, KEY_READ, hOpenKey)
    If Result <> 0 Then
        Err.Raise &H8004F610, _
                  TypeName(Me), _
                  "RegOpenKeyEx failed, system error " & CStr(Result)
    Else
        lngDataSize = 40
        S = Space$(lngDataSize)
        Result = RegQueryValueEx(hOpenKey, _
                                 StrPtr(ValueName), _
                                 0, _
                                 lngType, _
                                 StrPtr(S), _
                                 lngDataSize)
        RegCloseKey hOpenKey
        If Result <> 0 Then
            Err.Raise &H8004F612, _
                      TypeName(Me), _
                      "RegQueryValueEx failed, system error " & CStr(Result)
        Else
            GetFromRegistry = Trim$(Left$(S, lngDataSize - 1))
        End If
    End If
End Function

Private Sub Class_Initialize()
    Dim OS As OSVERSIONINFO

    OS.dwOSVersionInfoSize = Len(OS)
    GetVersionEx OS
    mvarPlatform = OS.dwPlatformId
End Sub
